#!/usr/bin/env python
# encoding: utf-8
"""
*given a sorted list of values, median sigma-clip values based on a window of values either side of each value (rolling window) and return the array mask*

:Author:
    David Young

:Date Created:
    January  1, 2021
"""
from builtins import object
import sys
import os
os.environ['TERM'] = 'vt100'
from fundamentals import tools


def rolling_window_sigma_clip(
        log,
        array,
        clippingSigma,
        windowSize):
    """*given a sorted list of values, median sigma-clip values based on a window of values either side of each value (rolling window) and return the array mask*

    **Key Arguments:**

    - `log` -- logger
    - `array` -- the array to clean up (arrays of length < 5 are not clipped but returned unmasked)
    - `clippingSigma` -- the minimum sigma to clip (using median-absolute distribution as sigma proxy)
    - `windowSize` -- the size of the window to use when calculating the median distribution (window of 11 will use 5 values each side of the value in question)

    **Usage:**

    ```python
    from fundamentals.stats import rolling_window_sigma_clip
    arrayMask = rolling_window_sigma_clip(
        log=self.log,
        array=myArray,
        clippingSigma=2.2,
        windowSize=11)

    ## JUST KEEP UNMASKED VALUES
    try:
        myArray = [e for e, m in zip(
            myArray, arrayMask) if m == False]
    except:
        myArray = []
    ```           
    """
    log.debug('starting the ``rolling_window_sigma_clip`` function')

    from astropy.stats import sigma_clip, mad_std

    midWindow = int((windowSize + 1) / 2)
    # ACCOMODATE SMALL LIST SIZES
    if len(array) < 5:
        return len(array) * [False]
    elif len(array) < windowSize:
        masked = sigma_clip(
            array, sigma_lower=clippingSigma, sigma_upper=clippingSigma, maxiters=7, cenfunc='median', stdfunc=mad_std)
        return list(masked.mask)
    startOfWindow = 0
    endOfWindow = windowSize
    maskedArray = []
    dataIndex = 0
    while len(array) >= endOfWindow:
        arrayWindow = array[startOfWindow:endOfWindow]
        startOfWindow += 1
        endOfWindow += 1
        masked = sigma_clip(
            arrayWindow, sigma_lower=clippingSigma, sigma_upper=clippingSigma, maxiters=7, cenfunc='median', stdfunc=mad_std)

        if dataIndex == 0:
            # 0,1,2...midWindow-1
            maskedArray += list(masked.mask)[0:midWindow]
            dataIndex += midWindow
        elif len(array) == endOfWindow - 1:
            # -midWindow...-2,-1
            maskedArray += list(masked.mask)[-midWindow:]
            dataIndex += midWindow
        else:
            maskedArray += [list(masked.mask)[midWindow - 1]]
            dataIndex += 1

    log.debug('completed the ``rolling_window_sigma_clip`` function')
    return maskedArray



from __future__ import print_function
from builtins import str
import os
import unittest
import shutil
import unittest
import yaml
from fundamentals.utKit import utKit
from fundamentals import tools
from os.path import expanduser
home = expanduser("~")

packageDirectory = utKit("").get_project_root()
settingsFile = packageDirectory + "/test_settings.yaml"
# settingsFile = home + \
#     "/git_repos/_misc_/settings/fundamentals/test_settings.yaml"

su = tools(
    arguments={"settingsFile": settingsFile},
    docString=__doc__,
    logLevel="DEBUG",
    options_first=False,
    projectName=None,
    defaultSettingsFile=False
)
arguments, settings, log, dbConn = su.setup()

# SETUP PATHS TO COMMON DIRECTORIES FOR TEST DATA
moduleDirectory = os.path.dirname(__file__)
pathToInputDir = moduleDirectory + "/input/"
pathToOutputDir = moduleDirectory + "/output/"

try:
    shutil.rmtree(pathToOutputDir)
except:
    pass
# COPY INPUT TO OUTPUT DIR
shutil.copytree(pathToInputDir, pathToOutputDir)

# Recursively create missing directories
if not os.path.exists(pathToOutputDir):
    os.makedirs(pathToOutputDir)


# xt-setup-unit-testing-files-and-folders
# xt-utkit-refresh-database

class test_rolling_window_sigma_clip(unittest.TestCase):

    def test_rolling_window_sigma_clip_function(self):
        import numpy as np
        import random
        normalDist = np.random.normal(loc=100, scale=3.0, size=[200])
        for i in range(1, 7):
            randNum = random.randint(1, 201)
            normalDist[randNum] = normalDist[randNum] * 4.5
        normalDist = list(normalDist)

        from fundamentals.stats import rolling_window_sigma_clip
        arrayMask = rolling_window_sigma_clip(
            log=log,
            array=normalDist,
            clippingSigma=2.2,
            windowSize=11)

        # JUST KEEP UNMASKED VALUES
        try:
            normalDist = [e for e, m in zip(
                normalDist, arrayMask) if m == False]
        except:
            normalDist = []

    def test_rolling_window_sigma_clip_function_exception(self):

        from fundamentals.stats import rolling_window_sigma_clip
        try:
            this = rolling_window_sigma_clip(
                log=log,
                settings=settings,
                fakeKey="break the code"
            )
            this.get()
            assert False
        except Exception as e:
            assert True
            print(str(e))

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function
